//************************************************************************
//
// ERROR CODES FOR ALL SYSTEMS in the SPINDLE CONTROLLER
//
// Errors can be generated by the following systems:
// High Level Spinstand Functions:ACUTRAC.CPP
// Central Input Output Handler:CIO.CPP
// DSP SERVO INTERFACE:SERVO.CPP-COMM.H
// Ramdisk Handler:RAMDISK.CPP
// Serial Protocol Handler:SERLPROT.CPP
// MultiTasking Kernel:TASK.CPP
//
//************************************************************************

#ifndef ERRORCODES__H
#define ERRORCODES__H

//----------------------------------------------------------------
// Error codes for Multi Tasking Kernel
//----------------------------------------------------------------

#define EVENT_NOERROR	0
#define EVENT_TIMEOUT	-96		//event has timed out
#define EVENT_OVERFLOW	-97		//too many events pending
#define EVENT_BUFFFULL	-98		//pipeline buffer full
#define EVENT_NOTASKS	-99		//no tasks to reschedule to
#define EVENT_TERMINATE	-100		//task needs to be canceled
#define EVENT_ABORTED	-101		//event was aborted by another task

//----------------------------------------------------------------
// Error Codes for Servo Controller
//----------------------------------------------------------------

#define SERVO_GOTACK		0		//error codes from WaitACK(), OK
#define SERVO_GOTTIMEOUT	-112	//TIMEOUT
#define SERVO_GOTNAK		-113   	//NAKked
#define SERVO_GOTACKTIMEOUT -114	//timed out waiting for ACK
#define SERVO_ERROR			-115
#define SERVO_UNKOWN_ERROR	-116 	//		  ???

//----------------------------------------------------------------
// Error Codes for Spindle Commands
//----------------------------------------------------------------

#define SPINDLE_OK			0		//everything is peachy keen
#define SPINDLE_ERROR		-128	//some sort of error
#define SPINDLE_NAK			-129	//command refused

//----------------------------------------------------------------
// Error Codes for Serial Protocol Handler
//----------------------------------------------------------------

/*
** return values from TelReceiveProtocol
*/

#define TEL_PROT_OK		0
#define TEL_PROT_TIMEOUT	-136
#define TEL_PROT_CHECKSUM	-137
#define TEL_PROT_BADHEADER	-138

//-----------------------------------------------------------------
// Error Codes for Ramdisk Driver
//-----------------------------------------------------------------

/*	Device Driver Errors	*/

#define RAMDISK_FILENOTFOUND		-64
#define RAMDISK_NOSECTORS			-65
#define RAMDISK_FILEOPEN			-66
#define RAMDISK_BADNAME				-67
#define RAMDISK_NODIRECTORIES		-68
#define RAMDISK_EOF					-69

//-----------------------------------------------------------------
// Error returned by misc commands
//-----------------------------------------------------------------

#define	GENERAL_OK					0
#define GENERAL_NAK					-80
#define GENERAL_ERROR				-81

//-----------------------------------------------------------------
// Error Codes for High Level Spindstand Functions
//-----------------------------------------------------------------

	/*	defines for ACTION_STATUS	*/
	/*	status error codes	*/

#define ACUTRAC_SUCCESS			0		//this is what you really want
#define ACUTRAC_ERROR			-1		//you get this when there is some error
#define ACUTRAC_BUSYERROR		-2		//you get this when spinstand doing something else
#define ACUTRAC_CONFIGFILE		-3		//you get this when something wrong with config file
#define ACUTRAC_SERVO_NAK		-4		//you get this when DSP doesn't want to do what you tell it
#define ACUTRAC_SERVO_TIMEOUT	-5		//you get this when comm to DSP fails
#define ACUTRAC_SERVO_UNKNOWN	-6		//you get this when it don't know
#define ACUTRAC_OUTOFRANGE		-7		//you get this when track is out of range
#define ACUTRAC_FILEERROR		-8		//you get this when RESET has problem with AFG file
#define ACUTRAC_FILEBAD			-9		//you get this when RESET can't find AFG file
#define ACUTRAC_RAMPTIMEOUT		-10		//you get this when ramp lifters don't move
#define ACUTRAC_LINEARTIMEOUT	-11		//you get this when X stage not moving
#define ACUTRAC_MOTIONTIMEOUT	-12		//you get this when rotary not moving
#define ACUTRAC_SPINDLETIMEOUT	-13		//you get this when motor not up to speed
#define ACUTRAC_USERABORT		-14		//you get this when stop button pushed
#define ACUTRAC_TIMEOUT			-15		//you get this when a semaphore times out
#define ACUTRAC_TERMINATE		-16		//you get this when task needs to terminate
#define ACUTRAC_NOTHANDLED		-17		//haven't figured out what to do in this case yet
#define ACUTRAC_LOWAIR			-18		//you get this when air pressure in system is low
#define ACUTRAC_NOINITINFO		-19		//no init file has been uploaded
#define ACUTRAC_AMPFAULT		-20		//you get this when the ampilfier faults
#define ACUTRAC_CLAMPTIMEOUT	-21		//you get this when the connector clamp times out

//-----------------------------------------------------------------
// Error Codes for Central I/O Handler
//-----------------------------------------------------------------

/*
	cio error codes
*/
#define	CIO_ABORT		-144	/*	device IO was interrupted by user	*/
#define	CIO_NO_HANDLES	-145	/*	NO MORE IOCB'S AVAILIABLE	*/
#define	CIO_NO_DEVICE	-146	/*	could not locate that device	*/
#define	CIO_WRONLY		-148	/*	Read was attempted on write only	*/
#define	CIO_INVALID		-149	/*	INVALID FUNCTION NUMBER	*/
#define	CIO_NOT_OPEN	-150	/*	IOCB NOT OPEN	*/
#define	CIO_INVLD_HNDL	-151	/*	INVALID HANDLE	*/
#define	CIO_RDONLY		-152	/*	Write was attempted on read only	*/
#define	CIO_END_OF_FILE	-153	/*	end of file encountered				*/
#define	CIO_TRUNCATED	-154	/*	NO EOL FOUND BEFORE END OF BUFFER	*/

#endif

